using System;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;

namespace IpPbxMPC
{
	/// <summary>
	/// Holds all IpPbxMPC configuration data.
	/// </summary>
	/// <remarks>
	/// This class will be serialized to disk to persist the configuration data.
	/// </remarks>
	[XmlRootAttribute("IpPbxMPCConfig", Namespace="http://www.lanphone.de/IpPbxMPCConfig", 
		 IsNullable = false),
	 XmlInclude(typeof(IpPbxBrowseAmp)), 
	 XmlInclude(typeof(IpPbxGeneric)),
     XmlInclude(typeof(IpPbxWinamp))
	]
	public class IpPbxMPCConfig
	{
		public enum ControlledApp
		{
			WindowsMediaPlayer,
			Winamp,
			BrowseAmp,
		};

		public enum IpPbxEvents
		{
			NonActiveLineKey,
			SpeedDialKeyState,
		};

		public IpPbxMPCConfig()
		{
			SpeedDialKeys = new ArrayList();
			AutoServerSelection = true;
			UseUserName = false;
			ControlledApplication = ControlledApp.Winamp;
		}

		/// <summary>
		/// Controlled Application. This member defines which type of application
		/// will be controlled by IpPbxMPC
		/// </summary>
		public ControlledApp ControlledApplication;

		/// <summary>
		/// Type of phone client event to react on.
		/// </summary>
		public IpPbxEvents EventToReactOn;

		/// <summary>
		/// If IpPbxMPC reacts on speed dial key states, this ArrayList contains
		/// the speed dial key ids to monitor
		/// </summary>
		[XmlArrayAttribute("SpeedDialKeys")]
		public ArrayList SpeedDialKeys;

		/// <summary>
		/// List of application specific configuration data. Each type of application
		/// can have it's own set of configuration data
		/// </summary>
		[XmlArrayAttribute("Apps")]
		public IpPbxApp[] Apps;

		/// <summary>
		/// if this member is true, IpPbxMPC will use the automatic PBX 
		/// server selection (via DHCP,DNS)
		/// </summary>
		public bool AutoServerSelection;

		/// <summary>
		/// If AutoServerSelection is false, this member defines the PBXServer to use
		/// </summary>
		public string ServerName;

		/// <summary>
		/// If AutoServerSelection is false, this member defines the fallback PBXServer to use
		/// </summary>
		public string FallbackServerName;

		/// <summary>
		/// If true, uses UserName member as PBX user name
		/// </summary>
		public bool UseUserName;

		/// <summary>
		/// If UseUserName is true, this member will be used as PBXUser name for login.
		/// Necessary when more than one PBX user have the same windows account assigned.
		/// </summary>
		public string UserName;

		/// <summary>
		/// Application configuration indexes
		/// Used to retrieve application specific configuration objects
		/// </summary>
		public IpPbxApp this[ControlledApp a]
		{
			get
			{
				if (  ((int)a)<Apps.GetLowerBound(0) 
					||((int)a)>Apps.GetUpperBound(0))
					throw new System.ArgumentOutOfRangeException();

				return Apps[(int)a];
			}
		}

		/// <summary>
		/// Helper function to determine if a speed dial key is among the 
		/// monitored keys
		/// </summary>
		/// <param name="idx">Speed dial key index</param>
		/// <returns>true, if given key state is in the SpeedDialKeys list, i.e.
		/// the state has to be monitored. False otherwise.</returns>
		public bool SpeedDialMonitored(int idx)
		{
			if (SpeedDialKeys == null)
				return false;
			return (SpeedDialKeys.BinarySearch(idx)>=0);
		}
	}
}
