﻿//-----------------------------------------------------------------------
// <copyright file="SampleAddin.cs" company="Swyx Solutions AG">
//     (c) Swyx Solutions AG
// </copyright>
//-----------------------------------------------------------------------
using System;
using System.Diagnostics;
using System.Globalization;
using System.Reflection;
using System.Runtime.InteropServices;
using CLMGRLib;

namespace CSharpAddIn
{
    /// <summary>
    /// Sample SwyxIt number resolver addin
    /// </summary>
    [ComVisible(true)]
    [Guid("373767E9-2953-42f9-98CA-63A232BC5B6F")]
    [ClassInterface(ClassInterfaceType.None)]
    public class SampleAddIn : IClientAddInLoader, IClientResolverAddIn, IFulltextSearchAddIn
    {
        /// <summary>
        /// trace source object for this type
        /// </summary>
        private static TraceSource traceSource = new TraceSource("SampleAddIn");

        /// <summary>
        /// Initializes a new instance of the SampleAddIn class
        /// </summary>
        public SampleAddIn()
        {
            traceSource.TraceInformation("SampleAddIn()");
        }

        #region IClientAddInLoader Members
        /// <summary>
        /// This method will be called after loading the PlugIn
        /// </summary>
        /// <param name="reserved">Reserved, must be zero</param>
        /// <returns>0 if successful, error code otherwise</returns>
        /// <remarks>If the PlugIn connects to the client line manager for receiving 
        /// events, it should establish the connection in response to "Initialize"
        /// Calling PubInit, PubRegisterUser and PubReleaseUser is not required for PlugIns.
        /// Registration should be done only by full clients, clients that intend to 
        /// provide a complete phone user interface like SwyxIt! itself.</remarks>
        public int Initialize(int reserved)
        {
            traceSource.TraceInformation("SampleAddIn.Initialize()");
            return 0;
        }

                /// <summary>
        /// This method returns the visible name of the PlugIn
        /// </summary>
        /// <param name="name">Plugin name</param>
        /// <returns>0 if successful, error code otherwise</returns>
        public int GetName(out string name)
        {
            name = "SampleAddin";
            traceSource.TraceInformation("SampleAddIn.GetName() returning {0}", name);
            return 0;
        }

        /// <summary>
        /// This method returns the visible version information of the PlugIn
        /// </summary>
        /// <param name="version">Plugin version</param>
        /// <returns>0 if successful, error code otherwise</returns>
        public int GetVersion(out string version)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            version = assembly.GetName().Version.ToString();
            traceSource.TraceInformation("SampleAddIn.GetVersion() returning {0}", version);
            return 0;
        }

        /// <summary>
        /// This method will be called before releasing the PlugIn
        /// </summary>
        /// <param name="reserved">Reserved, must be zero</param>
        /// <returns>0 if successful, error code otherwise</returns>
        /// <remarks>If the PlugIn connects to the client line manager for receiving 
        /// events, it should release the connection in response to "UnInitialize" 
        /// Calling PubInit, PubRegisterUser and PubReleaseUser is not required for 
        /// PlugIns.</remarks>
        public int UnInitialize(int reserved)
        {
            traceSource.TraceInformation("SampleAddIn.Uninitialize()");
            return 0;
        }

        #endregion

        #region IClientResolverAddIn Members

        /// <summary>
        /// This method finds a phone number to a name
        /// </summary>
        /// <param name="number">Number to search for. Format will be preferred number style as 
        /// returned by GetPreferredNumberStyle</param>
        /// <param name="name">Name to resolve</param>
        /// <returns>If the function succeeds, Name contains the name for the number and the
        /// return value is S_OK (0). If there's no name for the number, the name is empty and 
        /// the function returns S_FALSE (1)
        /// IF the number is empty the function returns E_INVALIDARG (0x80070057)</returns>
        public int ResolveNumber(string number, out string name)
        {
            name = string.Empty;
            int result = 1; // S_FALSE

            if (string.IsNullOrEmpty(number))
            {
                result = -2147024809; // E_INVALIDARG;
            }
            else
            {
                // This a just a sample, only resolve 4923147770
                if (string.Compare(number, "4923147770", StringComparison.OrdinalIgnoreCase) == 0)
                {
                    name = "Swyx Solutions";
                    result = 0;
                }
            }

            traceSource.TraceInformation(
                    "SampleAddIn.ResolveNumber({0}) name='{1}', result={2}",
                    number,
                    name,
                    result);

            return result;
        }

        /// <summary>
        /// Returns the preferred number style to be resolved
        /// </summary>
        /// <param name="preferredNumberStyle">Preferred number style</param>
        /// <returns>If the function succeeds, it returns S_OK (0)</returns>
        public int GetPreferredNumberStyle(out int preferredNumberStyle)
        {
            preferredNumberStyle = (int)PreferredNumberStyle.Plain;

            traceSource.TraceInformation(
                    "SampleAddIn.GetPreferredNumberStyle()  returning {0}", 
                    preferredNumberStyle);
            
            return 0;
        }

        #endregion

        #region IFulltextSearchAddIn Members

        /// <summary>
        /// This method will start a full text search in name or numbers in contacts
        /// </summary>
        /// <param name="searchText">Text fragment to be searched</param>
        /// <param name="enableSearchInNumbers">if set to TRUE, the search string will 
        /// be searched in numbers, too, BUT ONLY if it contains of solely digits. 
        /// If there is just one non-digit character in the search string, the search is 
        /// carried out only in the contact's names, as it will be if the variable 
        /// is set to FALSE</param>
        /// <param name="searchResultCollection">Search result collection to fill with results</param>
        /// <returns>If the function succeeds, the result is zero (S_OK)</returns>
        public int FulltextSearchInContacts(string searchText, bool enableSearchInNumbers, object searchResultCollection)
        {
            NameNumberSearchResultCollection resultCollection = (NameNumberSearchResultCollection)searchResultCollection;

            // This is a sample, we find "Swyx" only. 
            // Note: Swyxit always provides lower case text.
            if (   !String.IsNullOrEmpty(searchText)
                && searchText.Contains("swyx"))
            {
                NameNumberSearchResult result = new NameNumberSearchResult();
                result.Name = "Swyx Solutions AG (SampleAddIn result)";
                result.Number = "+4923147770";
                resultCollection.AddItem(result);

                result = new NameNumberSearchResult();
                result.Name = "Swyx Solutions AG Support (SampleAddIn result)";
                result.Number = "+492314777180";
                resultCollection.AddItem(result);
            }

            return 0;
        }

        #endregion
    }
}
